#pragma once

#include "TaskDialog.h"

class ErrorDialog : public Kerr::TaskDialog
{
public:

    ErrorDialog()
    {
        SetWindowTitle(L"Error Sample (page 1)");
        SetMainInstruction(L"An unexpected error occured. Please send feedback now!");
        SetVerificationText(L"Don't ask me again");
        SetContent(L"Error message goes here...");
        SetExpandedInformation(L"Stack trace goes here...");

        AddButton(L"Send Feedback\nI'm in a giving mood", Button_SendFeedback);
        AddButton(L"No Thanks\nI don't feel like being helpful", Button_NoThanks);

        m_config.dwFlags |= TDF_ALLOW_DIALOG_CANCELLATION | 
                            TDF_USE_COMMAND_LINKS | 
                            TDF_VERIFICATION_FLAG_CHECKED | 
                            TDF_EXPAND_FOOTER_AREA;
    }

private:

    enum
    {
        Button_SendFeedback = 101,
        Button_NoThanks
    };

    virtual void OnButtonClicked(int buttonId, bool& closeDialog)
    {
        switch (buttonId)
        {
            case Button_SendFeedback:
            {
                NavigatePage(m_sendFeedbackDialog);
                break;
            }
            case Button_NoThanks:
            case IDCANCEL:
            {
                closeDialog = true;
                break;
            }
            default:
            {
                ASSERT(false);
            }
        }
    }

    class SendFeedbackDialog : public Kerr::TaskDialog
    {
    public:

        SendFeedbackDialog()
        {
            SetWindowTitle(L"Error Sample (page 2)");
            SetContent(L"Copying your passwords  ;-)");

            m_config.dwFlags |= TDF_ALLOW_DIALOG_CANCELLATION | TDF_SHOW_PROGRESS_BAR | TDF_CALLBACK_TIMER;
        }

    private:

        enum
        {
            MaxRange = 5000
        };

        virtual void OnNavigated()
        {
            SetProgressBarRange(0, MaxRange);
        }

        virtual void OnButtonClicked(int /*buttonId*/, bool& closeDialog)
        {
            closeDialog = true;
        }

        virtual void OnTimer(DWORD milliseconds, bool& /*reset*/)
        {
            if (MaxRange >= milliseconds)
            {
                CString text;
                text.Format(L"Sending... %.f%%", static_cast<double>(milliseconds) / MaxRange * 100);
                SetMainInstruction(text.GetString());

                TRACE(L"%d\n", milliseconds);
                SetProgressBarPosition(milliseconds);
            }
            else
            {
                SetMainInstruction(L"Thanks for the feedback!");
                SetContent(L"Our developers will get right on that...");
                SetProgressBarPosition(MaxRange);
            }
        }

    };

    SendFeedbackDialog m_sendFeedbackDialog;

};
