//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   16 July 2006
//
//  Description:    Implements the main window for the sample application.
//
//*****************************************************************************

#pragma once

#include "CommonButtonsDialog.h"
#include "CounterDialog.h"
#include "ElevationRequiredDialog.h"
#include "EnableDisableDialog.h"
#include "ErrorDialog.h"
#include "IconsDialog.h"
#include "ProgressDialog.h"
#include "ProgressEffectsDialog.h"
#include "TimerDialog.h"
#include "UpdateTextDialog.h"

class MainWindow : public Kerr::TaskDialog
{
public:

    MainWindow()
    {
        SetWindowTitle(L"TaskDialog Samples");
        SetMainInstruction(L"Pick a sample to try:");
        SetFooter(L"Created by <a href=\"http://weblogs.asp.net/kennykerr/\">Kenny Kerr</a>");

        m_config.dwFlags |= TDF_ALLOW_DIALOG_CANCELLATION | 
                            TDF_USE_COMMAND_LINKS | 
                            TDF_CAN_BE_MINIMIZED |
                            TDF_ENABLE_HYPERLINKS;

        AddButton(L"Common Buttons Sample",     Button_CommonButtons);
        AddButton(L"Counter Sample",            Button_Counter);
        AddButton(L"Elevation Requied Sample",  Button_ElevationRequied);
        AddButton(L"Enable/Disable Sample",     Button_EnableDisable);
        AddButton(L"Error Sample",              Button_Error);
        AddButton(L"Icons Sample",              Button_Icons);
        AddButton(L"Progress Sample",           Button_Progress);
        AddButton(L"Progress Effects Sample",   Button_ProgressEffects);
        AddButton(L"Timer Sample",              Button_Timer);
        AddButton(L"Update Text Sample",        Button_UpdateText);
    }

private:

    enum
    {
        Button_CommonButtons = 101,
        Button_Counter,
        Button_ElevationRequied,
        Button_EnableDisable,
        Button_Error,
        Button_Icons,
        Button_Progress,
        Button_ProgressEffects,
        Button_Timer,
        Button_UpdateText
    };

    virtual void OnHyperlinkClicked(PCWSTR url)
    {
        ASSERT(0 != url);

        ::ShellExecute(m_hWnd,
                       L"open",
                       url,
                       0,
                       0,
                       SW_SHOWNORMAL);
    }

    virtual void OnButtonClicked(int buttonId, 
                                 bool& closeDialog)
    {
        switch (buttonId)
        {
            case Button_CommonButtons:
            {
                CommonButtonsDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_Counter:
            {
                CounterDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_ElevationRequied:
            {
                ElevationRequiedDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_EnableDisable:
            {
                EnableDisableDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_Error:
            {
                ErrorDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_Icons:
            {
                IconsDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_Progress:
            {
                ProgressDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_ProgressEffects:
            {
                ProgressEffectsDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_Timer:
            {
                TimerDialog dialog;
                dialog.DoModal();
                break;
            }
            case Button_UpdateText:
            {
                UpdateTextDialog dialog;
                dialog.DoModal();
                break;
            }
            case IDCANCEL:
            {
                closeDialog = true;
                break;
            }
            default:
            {
                ASSERT(false);
            }
        }
    }

};
