#pragma once

#include "TaskDialog.h"

class ProgressEffectsDialog : public Kerr::TaskDialog
{
public:

    ProgressEffectsDialog() :
        m_state(PBST_NORMAL),
        m_marquee(false)
    {
        SetWindowTitle(L"Progress Effects");

        AddButton(L"Cycle State", Button_State);
        AddButton(L"Toggle Mode", Button_Mode);

        m_config.dwFlags |= TDF_ALLOW_DIALOG_CANCELLATION | TDF_SHOW_PROGRESS_BAR;
    }

private:

    enum
    {
        Button_State = 101,
        Button_Mode,
    };

    virtual void OnDialogConstructed()
    {
        UpdateMainInstruction();
        SetProgressBarPosition(25);
    }

    virtual void OnButtonClicked(int buttonId, bool& closeDialog)
    {
        switch (buttonId)
        {
            case Button_State:
            {
                CycleState();
                break;
            }
            case Button_Mode:
            {
                ToggleMode();
                break;
            }
            case IDCANCEL:
            {
                closeDialog = true;
                break;
            }
            default:
            {
                ASSERT(false);
            }
        }
    }

    void UpdateMainInstruction()
    {
        CString text;

        switch (m_state)
        {
            case PBST_NORMAL:
            {
                text.Append(L"State: Normal\n");
                break;
            }
            case PBST_PAUSED:
            {
                text.Append(L"State: Paused\n");
                break;
            }
            case PBST_ERROR:
            {
                text.Append(L"State: Error\n");
                break;
            }
            default:
            {
                ASSERT(false);
            }
        }

        text.AppendFormat(L"Marquee: %s", m_marquee ? L"True" : L"False");
        SetMainInstruction(text.GetString());
    }

    void CycleState()
    {
        switch (m_state)
        {
            case PBST_NORMAL:
            {
                m_state = PBST_PAUSED;
                break;
            }
            case PBST_PAUSED:
            {
                m_state = PBST_ERROR;
                break;
            }
            case PBST_ERROR:
            {
                m_state = PBST_NORMAL;
                break;
            }
            default:
            {
                ASSERT(false);
            }
        }

        SetProgressBarState(m_state);
        UpdateMainInstruction();
    }

    void ToggleMode()
    {
        m_marquee = !m_marquee;
        SetMarqueeProgressBar(m_marquee);

        if (m_marquee)
        {
            // Start marquee at top speed
            SetProgressBarMarquee(true, 0);
        }
        else
        {
            SetProgressBarPosition(25);
        }

        UpdateMainInstruction();
    }

    int m_state;
    bool m_marquee;

};
