//*****************************************************************************
//
//  Author:         Kenny Kerr
//  Date created:   16 July 2006
//
//  Description:    Implements a C++ wrapper for the TaskDialogIndirect 
//                  function.
//
//*****************************************************************************

#pragma once

namespace Kerr
{
    class TaskDialog : public CWindow
    {
    public:

        explicit TaskDialog();
        virtual ~TaskDialog() {}

        // Set text captions
        void SetWindowTitle(ATL::_U_STRINGorID text);
        void SetMainInstruction(ATL::_U_STRINGorID text);
        void SetContent(ATL::_U_STRINGorID text);
        void SetVerificationText(ATL::_U_STRINGorID text);
        void SetExpandedInformation(ATL::_U_STRINGorID text);
        void SetExpandedControlText(ATL::_U_STRINGorID text);
        void SetCollapsedControlText(ATL::_U_STRINGorID text);
        void SetFooter(ATL::_U_STRINGorID text);

        // Set icons
        void SetMainIcon(HICON handle);
        void SetMainIcon(ATL::_U_STRINGorID resource);
        void SetFooterIcon(HICON handle);
        void SetFooterIcon(ATL::_U_STRINGorID resource);

        void AddButton(ATL::_U_STRINGorID text,
                       int id);

        void AddRadioButton(ATL::_U_STRINGorID text,
                            int id);

        HRESULT DoModal(HWND parent = ::GetActiveWindow());
        int GetSelectedButtonId() const;
        int GetSelectedRadioButtonId() const;
        bool VerificiationChecked() const;

        // Messages
        void ClickButton(int buttonId);
        void ClickRadioButton(int buttonId);
        void ClickVerification(bool checked, bool setKeyFocus);
        void EnableButton(int buttonId, bool enable = true);
        void EnableRadioButton(int buttonId, bool enable = true);
        void SetMarqueeProgressBar(bool marquee = true);
        void SetProgressBarState(int state);
        void SetProgressBarRange(WORD minRange = 0, WORD maxRange = 100);
        void SetProgressBarPosition(int position);
        void SetProgressBarMarquee(bool marquee, DWORD speed);
        void SetButtonElevationRequired(int buttonId, bool required = true);
        void NavigatePage(TaskDialog& newDialog);

    protected:

        // Events
        virtual void OnDialogConstructed() {}
        virtual void OnHyperlinkClicked(PCWSTR /*url*/) {}
        virtual void OnButtonClicked(int /*buttonId*/, bool& closeDialog) { closeDialog = true; }
        virtual void OnRadioButtonClicked(int /*buttonId*/) {}
        virtual void OnVerificationClicked(bool /*checked*/) {}
        virtual void OnHelp() {}
        virtual void OnExpandoButtonClicked(bool /*expanded*/) {}
        virtual void OnTimer(DWORD /*milliseconds*/, bool& /*reset*/) {}
        virtual void OnNavigated() {}

        static HRESULT CALLBACK Callback(HWND handle, 
                                         UINT notification, 
                                         WPARAM wParam, 
                                         LPARAM lParam, 
                                         LONG_PTR data);

        TASKDIALOGCONFIG m_config;
        CAtlArray<TASKDIALOG_BUTTON> m_buttons;
        CAtlArray<TASKDIALOG_BUTTON> m_radioButtons;
        int m_selectedButtonId;
        int m_selectedRadioButtonId;
        BOOL m_verificationChecked;
    };
}

Kerr::TaskDialog::TaskDialog() :
    m_selectedButtonId(0),
    m_selectedRadioButtonId(0),
    m_verificationChecked(FALSE)
{
    ::ZeroMemory(&m_config, 
                 sizeof (TASKDIALOGCONFIG));

    m_config.cbSize = sizeof (TASKDIALOGCONFIG);
    m_config.hInstance = ATL::_AtlBaseModule.GetResourceInstance();
    m_config.pfCallback = Callback;
    m_config.lpCallbackData = reinterpret_cast<LONG_PTR>(this);
    m_config.dwFlags = TDF_POSITION_RELATIVE_TO_WINDOW;
}

void Kerr::TaskDialog::SetWindowTitle(ATL::_U_STRINGorID text)
{
    if (0 == m_hWnd)
    {
        m_config.pszWindowTitle = text.m_lpstr;
    }
    else if (IS_INTRESOURCE(text.m_lpstr))
    {
        CString string;

        // Since we know that text is actually a resource Id we can ignore the pointer truncation warning.
        #pragma warning(push)
        #pragma warning(disable: 4311)

        VERIFY(string.LoadString(m_config.hInstance,
                                 reinterpret_cast<UINT>(text.m_lpstr)));

        #pragma warning(pop)

        VERIFY(SetWindowText(string));
    }
    else
    {
        VERIFY(SetWindowText(text.m_lpstr));
    }
}

void Kerr::TaskDialog::SetMainInstruction(ATL::_U_STRINGorID text)
{
    if (0 == m_hWnd)
    {
        m_config.pszMainInstruction = text.m_lpstr;
    }
    else
    {
        SendMessage(TDM_SET_ELEMENT_TEXT,
                    TDE_MAIN_INSTRUCTION,
                    reinterpret_cast<LPARAM>(text.m_lpstr));
    }
}

void Kerr::TaskDialog::SetContent(ATL::_U_STRINGorID text)
{
    if (0 == m_hWnd)
    {
        m_config.pszContent = text.m_lpstr;
    }
    else
    {
        SendMessage(TDM_SET_ELEMENT_TEXT,
                    TDE_CONTENT,
                    reinterpret_cast<LPARAM>(text.m_lpstr));
    }
}

void Kerr::TaskDialog::SetVerificationText(ATL::_U_STRINGorID text)
{
    m_config.pszVerificationText = text.m_lpstr;
}

void Kerr::TaskDialog::SetExpandedInformation(ATL::_U_STRINGorID text)
{
    if (0 == m_hWnd)
    {
        m_config.pszExpandedInformation = text.m_lpstr;
    }
    else
    {
        SendMessage(TDM_SET_ELEMENT_TEXT,
                    TDE_EXPANDED_INFORMATION,
                    reinterpret_cast<LPARAM>(text.m_lpstr));
    }
}

void Kerr::TaskDialog::SetExpandedControlText(ATL::_U_STRINGorID text)
{
    m_config.pszExpandedControlText = text.m_lpstr;
}

void Kerr::TaskDialog::SetCollapsedControlText(ATL::_U_STRINGorID text)
{
    m_config.pszCollapsedControlText = text.m_lpstr;
}

void Kerr::TaskDialog::SetFooter(ATL::_U_STRINGorID text)
{
    if (0 == m_hWnd)
    {
        m_config.pszFooter = text.m_lpstr;
    }
    else
    {
        SendMessage(TDM_SET_ELEMENT_TEXT,
                    TDE_FOOTER,
                    reinterpret_cast<LPARAM>(text.m_lpstr));
    }
}

void Kerr::TaskDialog::SetMainIcon(HICON handle)
{
    if (0 == m_hWnd)
    {
        m_config.hMainIcon = handle;
        m_config.dwFlags |= TDF_USE_HICON_MAIN;
    }
    else
    {
        ASSERT(TDF_USE_HICON_MAIN & m_config.dwFlags);

        SendMessage(TDM_UPDATE_ICON,
                    TDIE_ICON_MAIN,
                    reinterpret_cast<LPARAM>(handle));
    }
}

void Kerr::TaskDialog::SetMainIcon(ATL::_U_STRINGorID resource)
{
    if (0 == m_hWnd)
    {
        m_config.pszMainIcon = resource.m_lpstr;
        m_config.dwFlags &= ~TDF_USE_HICON_MAIN;
    }
    else
    {
        ASSERT(0 == (TDF_USE_HICON_MAIN & m_config.dwFlags));

        SendMessage(TDM_UPDATE_ICON,
                    TDIE_ICON_MAIN,
                    reinterpret_cast<LPARAM>(resource.m_lpstr));
    }
}

void Kerr::TaskDialog::SetFooterIcon(HICON handle)
{
    if (0 == m_hWnd)
    {
        m_config.hFooterIcon = handle;
        m_config.dwFlags |= TDF_USE_HICON_FOOTER;
    }
    else
    {
        ASSERT(TDF_USE_HICON_FOOTER & m_config.dwFlags);

        SendMessage(TDM_UPDATE_ICON,
                    TDIE_ICON_FOOTER,
                    reinterpret_cast<LPARAM>(handle));
    }
}

void Kerr::TaskDialog::SetFooterIcon(ATL::_U_STRINGorID resource)
{
    if (0 == m_hWnd)
    {
        m_config.pszFooterIcon = resource.m_lpstr;
        m_config.dwFlags &= ~TDF_USE_HICON_FOOTER;
    }
    else
    {
        ASSERT(0 == (TDF_USE_HICON_FOOTER & m_config.dwFlags));

        SendMessage(TDM_UPDATE_ICON,
                    TDIE_ICON_FOOTER,
                    reinterpret_cast<LPARAM>(resource.m_lpstr));
    }
}

void Kerr::TaskDialog::AddButton(ATL::_U_STRINGorID text,
                                 int id)
{
    size_t index = m_buttons.Add();
    m_buttons[index].pszButtonText = text.m_lpstr;
    m_buttons[index].nButtonID = id;
}

void Kerr::TaskDialog::AddRadioButton(ATL::_U_STRINGorID text,
                                      int id)
{
    size_t index = m_radioButtons.Add();
    m_radioButtons[index].pszButtonText = text.m_lpstr;
    m_radioButtons[index].nButtonID = id;
}

HRESULT Kerr::TaskDialog::DoModal(HWND parent)
{
    ASSERT(0 == m_hWnd);

    m_config.hwndParent = parent;
    m_config.pButtons = m_buttons.GetData();
    m_config.cButtons = static_cast<UINT>(m_buttons.GetCount());
    m_config.pRadioButtons = m_radioButtons.GetData();
    m_config.cRadioButtons = static_cast<UINT>(m_radioButtons.GetCount());

    return ::TaskDialogIndirect(&m_config,
                                &m_selectedButtonId,
                                &m_selectedRadioButtonId,
                                &m_verificationChecked);
}

int Kerr::TaskDialog::GetSelectedButtonId() const
{
    return m_selectedButtonId;
}

int Kerr::TaskDialog::GetSelectedRadioButtonId() const
{
    return m_selectedRadioButtonId;
}

bool Kerr::TaskDialog::VerificiationChecked() const
{
    return FALSE != m_verificationChecked;
}

void Kerr::TaskDialog::ClickButton(int buttonId)
{
    SendMessage(TDM_CLICK_BUTTON,
                buttonId);
}

void Kerr::TaskDialog::ClickRadioButton(int buttonId)
{
    SendMessage(TDM_CLICK_RADIO_BUTTON,
                buttonId);
}

void Kerr::TaskDialog::ClickVerification(bool checked, 
                                         bool setKeyFocus)
{
    SendMessage(TDM_CLICK_VERIFICATION,
                checked,
                setKeyFocus);
}

void Kerr::TaskDialog::EnableButton(int buttonId, 
                                    bool enable)
{
    SendMessage(TDM_ENABLE_BUTTON,
                buttonId,
                enable);
}

void Kerr::TaskDialog::EnableRadioButton(int buttonId, 
                                         bool enable)
{
    SendMessage(TDM_ENABLE_RADIO_BUTTON,
                buttonId,
                enable);
}

void Kerr::TaskDialog::SetMarqueeProgressBar(bool marquee)
{
    SendMessage(TDM_SET_MARQUEE_PROGRESS_BAR,
                marquee);
}

void Kerr::TaskDialog::SetProgressBarMarquee(bool marquee,
                                             DWORD milliseconds)
{
    SendMessage(TDM_SET_PROGRESS_BAR_MARQUEE,
                marquee,
                milliseconds);
}

void Kerr::TaskDialog::SetProgressBarState(int state)
{
    SendMessage(TDM_SET_PROGRESS_BAR_STATE,
                state);
}

void Kerr::TaskDialog::SetProgressBarPosition(int position)
{
    SendMessage(TDM_SET_PROGRESS_BAR_POS,
                position);
}

void Kerr::TaskDialog::SetProgressBarRange(WORD minRange, 
                                           WORD maxRange)
{
    SendMessage(TDM_SET_PROGRESS_BAR_RANGE,
                0,
                MAKELPARAM(minRange, maxRange));
}

void Kerr::TaskDialog::SetButtonElevationRequired(int buttonId, 
                                                  bool required)
{
    SendMessage(TDM_SET_BUTTON_ELEVATION_REQUIRED_STATE,
                buttonId,
                required);
}

void Kerr::TaskDialog::NavigatePage(TaskDialog& newDialog)
{
    ASSERT(0 == newDialog.m_hWnd);

    SendMessage(TDM_NAVIGATE_PAGE,
                0,
                reinterpret_cast<LPARAM>(&newDialog.m_config));
}

HRESULT Kerr::TaskDialog::Callback(HWND handle, 
                                   UINT notification, 
                                   WPARAM wParam, 
                                   LPARAM lParam, 
                                   LONG_PTR data)
{
    ASSERT(0 != handle);
    ASSERT(0 != data);

    TaskDialog* pThis = reinterpret_cast<TaskDialog*>(data);
    HRESULT result = S_OK;

    switch (notification)
    {
        case TDN_NAVIGATED:
        {
            pThis->OnNavigated();
            break;
        }
        case TDN_BUTTON_CLICKED:
        {
            bool closeDialog = false;
            pThis->OnButtonClicked(static_cast<int>(wParam), closeDialog);
            result = !closeDialog;
            break;
        }
        case TDN_HYPERLINK_CLICKED:
        {
            pThis->OnHyperlinkClicked(reinterpret_cast<PCWSTR>(lParam));
            break;
        }
        case TDN_TIMER:
        {
            ASSERT(UINT_MAX >= wParam);
            bool reset = false;
            pThis->OnTimer(static_cast<DWORD>(wParam), reset);
            result = reset;
            break;
        }
        case TDN_DESTROYED:
        {
            pThis->Detach();
            break;
        }
        case TDN_RADIO_BUTTON_CLICKED:
        {
            pThis->OnRadioButtonClicked(static_cast<int>(wParam));
            break;
        }
        case TDN_CREATED:
        {
            break;
        }
        case TDN_DIALOG_CONSTRUCTED:
        {
            pThis->Attach(handle);
            pThis->OnDialogConstructed();
            break;
        }
        case TDN_VERIFICATION_CLICKED:
        {
            pThis->OnVerificationClicked(0 != wParam);
            break;
        }
        case TDN_HELP:
        {
            pThis->OnHelp();
            break;
        }
        case TDN_EXPANDO_BUTTON_CLICKED:
        {
            pThis->OnExpandoButtonClicked(0 != wParam);
            break;
        }
        default:
        {
            TRACE(L"TaskDialog::Callback -- Unhandled notification encountered\n");
            break;
        }

    }

    return result;
}
